# encoding: utf-8
%w(
  exceptions
  scope
  verification
).each { |name| require_relative "verifier/#{ name }" }

# Defines {#verify} method to check variables
#
# @example
#   class Greeter
#     include Verifier
#
#     def hello(name)
#       verify(:name, :long_enough) { count > 1 }
#       "Hello, #{ name }!"
#     end
#   end
#
#   greeter = Greeter.new
#   greeter.hello "Ali"
#   # => "Hello, Ali!"
#
#   greeter.hello "X"
#   # => <Verifier::Invalid ...>
module Verifier

  # Runs a verification
  #
  # @param [Symbol] method
  #   the name of the method to be verified
  # @param [Symbol] name
  #   the name of verification
  # @param [Proc] block
  #
  # @yield block in the scope of a value returned by the method
  #
  # @raise  [Verifier::MethodNotDefined]
  #   if the method not defined
  # @raise  [Verifier::MethodFails]
  #   it calling the method raises a +StandardError+
  # @raise  [Verifier::Invalid]
  #   if the block fails or returns a value whose negation is truthy
  #
  # @return [undefined]
  #
  # @api public
  def verify(method, name, &block)
    Verification.run(self, method, name, &block)
  end
end
