# encoding: utf-8
require "extlib"

module Verifier

  # Provides the scope for I18n translations of verification errors
  #
  # @api private
  class Scope

    # Initializes a new instance
    #
    # @param [Object] object
    #   the object whose method should be verified
    # @param [Symbol] method
    #   the name of the method to be verified
    #
    # @return [undefined]
    def initialize(object, method)
      @object, @method = object, method
    end

    # Returns a scope for all translations of {#object} verification errors
    #
    # @return [Array<Symbol>]
    def short
      @short ||= [:verifier, name, type]
    end

    # Returns a scope for translations of {#method} verification errors
    #
    # @return [Array<Symbol>]
    def full
      @full ||= short + [method.to_sym]
    end

    # Returns a tag (# or .) for the method
    #
    # @return [<String>]
    def tag
      (type == :class) ? "." : "#"
    end

    private

    attr_reader :object, :method

    def type
      @type ||= object.is_a?(Class) ? :class : :instance
    end

    def name
      @name ||= object_class_name.split("::").map(&:snake_case).join("/").to_sym
    end

    def object_class_name
      object.class.name
    end
  end
end
